#!/usr/bin/perl -w
#
# $Id: check_iostat,v 1.3 2004/06/16 22:12:10 hmann Exp $
#
# check_iostat checks the IO rates on all or a selected disk
# applies thresholds to the values found and returns a Nagios
# alarm if needed.
#
# No warrenty of any kind is implied or granted.
#
# Usage: "See usage() below"
#
# Change Log
#----------------
# 7-Feb-2004 - Harper Mann
#  Initial revision
# 11-Feb-2004 - Harper Mann
#  Added -k kilobytes and -p performance switches
# 2-Jun-2004 - Scott Parris
#  Added timer interval check with iostat so the
#  Stats are for a short time period rather than
#  from boot.
#
use strict;

my @vals = undef;
my @skips = undef;
my @lines = undef;

my $lo_rd_warn = -1;
my $lo_wt_warn = -1;
my $lo_tt_warn = -1;

my $hi_rd_warn = -1;
my $hi_wt_warn = -1;
my $hi_tt_warn = -1;

my $lo_rd_crit = -1;
my $lo_wt_crit = -1;
my $lo_tt_crit = -1;

my $hi_rd_crit = -1;
my $hi_wt_crit = -1;
my $hi_tt_crit = -1;
 
my $perf = undef;
my $kb = undef;
my $dev = undef;
my $debug = 0;
my $res = undef;
my $IOSTAT = "iostat"; 

use Getopt::Long;
use vars qw($opt_V $opt_r $opt_R $opt_w $opt_W $opt_t $opt_T $opt_d $opt_s $opt_k $opt_D $opt_p $opt_h);
use vars qw($PROGNAME);
use lib "/usr/local/nagios/libexec"  ;
use utils qw($TIMEOUT %ERRORS &print_revision &support &usage);

sub print_help ();
sub print_usage ();

$PROGNAME = "check_iostat";

Getopt::Long::Configure('bundling');
my $status = GetOptions
        ("V"   => \$opt_V, "Version"         => \$opt_V,
         "r=s" => \$opt_r, "readwarning=s"   => \$opt_r,
         "R=s" => \$opt_R, "readcritical=s"  => \$opt_R,
         "w=s" => \$opt_w, "writewarning=s"  => \$opt_w,
         "W=s" => \$opt_W, "writecritical=s" => \$opt_W,
         "t=s" => \$opt_t, "waitwarning=s"   => \$opt_t,
         "T=s" => \$opt_T, "waitcritical=s"  => \$opt_T,
         "d=s" => \$opt_d, "devices=s"       => \$opt_d,
         "s=s" => \$opt_s, "skip=s"          => \$opt_s,
         "k"   => \$opt_k, "kilobytes"       => \$opt_k,
         "D"   => \$opt_D, "debug"           => \$opt_D,
         "p"   => \$opt_p, "performance"     => \$opt_p,
         "h"   => \$opt_h, "help"            => \$opt_h);

if ($status == 0)
{
        print_usage() ;
        exit $ERRORS{'OK'};
}

# Debug switch
if ($opt_D) {
	$debug = 1;
}

if ($opt_V) {
        print_revision($PROGNAME,'$Revision: 1.4 $'); #'
        exit $ERRORS{'OK'};
}

if ($opt_h) {print_help(); exit $ERRORS{'OK'};}

# Options checking
# Read Warning
if ($opt_r) {
	@vals = split /:/, $opt_r;
	($vals[0]) || usage("Invalid value: low read warning: $opt_r\n");
	($vals[1]) || usage("Invalid value: high read warning: $opt_r\n");
	$lo_rd_warn = $vals[0] if ($vals[0] =~ /^[0-9]+\.[0-9]+$/);
	$hi_rd_warn = $vals[1] if ($vals[1] =~ /^[0-9]+\.[0-9]+$/);
	($lo_rd_warn > -1) || 
		usage("Invalid value: low read warning: $opt_r\n");
	($hi_rd_warn > -1) || 
		usage("Invalid value: high read warning: $opt_r\n");
} else { print "No read warning defined\n" if $debug }

# Read Critical
if ($opt_R) {
	@vals = split /:/, $opt_R;
	($vals[0]) || usage("Invalid value: low read critical: $opt_R\n");
	($vals[1]) || usage("Invalid value: high read critical: $opt_R\n");
	$lo_rd_crit = $vals[0] if ($vals[0] =~ /^[0-9]+\.[0-9]+$/);
	$hi_rd_crit = $vals[1] if ($vals[1] =~ /^[0-9]+\.[0-9]+$/);
	($lo_rd_crit > -1) || 
		usage("Invalid value: low read critical: $opt_R\n");
	($hi_rd_crit > -1) || 
		usage("Invalid value: high read critical: $opt_R\n");
} else { print "No read critical defined\n" if $debug }

# Write Warning
if ($opt_w) {
	@vals = split /:/, $opt_w;
	($vals[0]) || usage("Invalid value: low write warning: $opt_w\n");
	($vals[1]) || usage("Invalid value: high write warning: $opt_w\n");
	$lo_wt_warn = $vals[0] if ($vals[0] =~ /^[0-9]+\.[0-9]+$/);
	$hi_wt_warn = $vals[1] if ($vals[1] =~ /^[0-9]+\.[0-9]+$/);
	($lo_wt_warn > -1) || 
		usage("Invalid value: low write warning: $opt_w\n");
	($hi_wt_warn > -1) || 
		usage("Invalid value: high write warning: $opt_w\n");
} else { print "No write warning defined\n" if $debug }

# Write Critical
if ($opt_W) {
	@vals = split /:/, $opt_W;
	($vals[0]) || usage("Invalid value: low write critical: $opt_W\n");
	($vals[1]) || usage("Invalid value: high write critical: $opt_W\n");
	$lo_wt_crit = $vals[0] if ($vals[0] =~ /^[0-9]+\.[0-9]+$/);
	$hi_wt_crit = $vals[1] if ($vals[1] =~ /^[0-9]+\.[0-9]+$/);
	($lo_wt_crit > -1) || 
		usage("Invalid value: low write critical: $opt_W\n");
	($hi_wt_crit > -1) || 
		usage("Invalid value: high write critical: $opt_W\n");
} else { print "No write critical defined\n" if $debug }

# Await Warning
if ($opt_t) {
	@vals = split /:/, $opt_t;
	($vals[0]) || usage("Invalid value: low await warning: $opt_t\n");
	($vals[1]) || usage("Invalid value: high await warning: $opt_t\n");
	$lo_tt_warn = $vals[0] if ($vals[0] =~ /^[0-9]+\.[0-9]+$/);
	$hi_tt_warn = $vals[1] if ($vals[1] =~ /^[0-9]+\.[0-9]+$/);
	($lo_tt_warn > -1) || 
		usage("Invalid value: low await warning: $opt_t\n");
	($hi_tt_warn > -1) || 
		usage("Invalid value: high await warning: $opt_t\n");
} else { print "No await warning defined\n" if $debug }

# Await Critical
if ($opt_T) {
	@vals = split /:/, $opt_T;
	($vals[0]) || usage("Invalid value: low await critical: $opt_T\n");
	($vals[1]) || usage("Invalid value: high await critical: $opt_T\n");
	$lo_tt_crit = $vals[0] if ($vals[0] =~ /^[0-9]+\.[0-9]+$/);
	$hi_tt_crit = $vals[1] if ($vals[1] =~ /^[0-9]+\.[0-9]+$/);
	($lo_tt_crit > -1) || 
		usage("Invalid value: low await critical: $opt_T\n");
	($hi_tt_crit > -1) || 
		usage("Invalid value: high await critical: $opt_T\n");
} else { print "No await critical defined\n" if $debug }

# Want report in kilobytes or blocks?
if ($opt_k) {$kb = 1}

# Want report in kilobytes or blocks?
if ($opt_p) {$perf = 1}

# Device list if any
if ($opt_d) {
	$dev = $opt_d;
	print "\nDevice list: $dev\n" if $debug;
	$dev =~ s/,/ /g;
}

# Skip list if any
if ($opt_s) {
	$opt_s =~ s/\/dev\///; 		# chop /dev/ if found
	@skips = split /,/, $opt_s;
}


# Run iostat on all disks or just specified list.  See iostat man.
# If $def is defined, only devices on list will be checked.  -s skips 
# devices on list.  Both are comma (,) separated
#

if ($dev) {
	print "Command: $IOSTAT 10 2 -k -d -x $dev\n" if $debug;
	$res = qx/$IOSTAT 10 2 -k -d -x $dev/;
	my @stats = split(/\n\n/, $res);
	$res = $stats[2];
} else {
	print "Command: $IOSTAT 10 2 -k -d -x\n" if $debug;
	$res = qx/$IOSTAT 10 2 -k -d -x/;
	my @stats = split(/\n\n/, $res);
	$res = $stats[2];
}
print "\niostat output:\n$res" if $debug;

# Split $res into an array of lines
# $vals[7] is kb reads, $vals[8] is kb writes
# Loop through all devices found; one each line
@lines = split /\n/, $res;

my $dv = 0;
my $rval = 0;
my $wval = 0;
my $tval = 0;
my @badlist = ();
my @goodlist = ();
my @perflist = ();
my $worst = 0; # Status OK

OUTER: 
for my $line (@lines) {
	next if !($line =~ /^\//);

	# Check skip list
INNER: 		
	for my $skip (@skips) {
		next if !$skip;
		if ($line =~ /$skip /) {
			print "skipping: $skip\n" if $debug;
			next OUTER 
		}
	}

	# Split the values off white space
	# Blocks read is position 5, write is 6, kb read is 7, kb write is 8
	@vals = split /\s+/, $line;

	# Device
	$dv = $vals[0];
	$dv =~ s/\/dev\///;

	# Is we kb or blocks?
	$kb ? 
		($rval = $vals[7], $wval = $vals[8]) :
		($rval = $vals[5], $wval = $vals[6]);

	# Await vals
	$tval = $vals[11];

	# Keep a list of values for an OK exit
	push @goodlist, "($dv rd: $rval wr: $wval aw: $tval)";

	# Keep a list of values for performance output
	push @perflist, "$dv $rval $wval $tval";

	# Okay, we have everything so run the checks
	# Criticals
	if ($hi_rd_crit > -1 && $rval > $hi_rd_crit) {
		push @badlist, 
			"$dv reads high $rval (critical)";
		$worst = $ERRORS{'CRITICAL'};
	}
	if ($hi_wt_crit > -1 && $wval > $hi_wt_crit) {
		push @badlist, 
			"$dv writes high $wval (critical)";
		$worst = $ERRORS{'CRITICAL'};
	}
	if ($hi_tt_crit > -1 && $tval > $hi_tt_crit) {
		push @badlist, 
			"$dv await high $tval (critical)";
		$worst = $ERRORS{'CRITICAL'};
	}
	if ($lo_rd_crit > -1 && $rval < $lo_rd_crit) {
		push @badlist, 
			"$dv reads low $rval (critical)";
		$worst = $ERRORS{'CRITICAL'};
	}
	if ($lo_wt_crit > -1 && $wval < $lo_wt_crit) {
		push @badlist, 
			"$dv writes low $wval (critical)";
		$worst = $ERRORS{'CRITICAL'};
	}
	if ($lo_tt_crit > -1 && $tval < $lo_tt_crit) {
		push @badlist, 
			"$dv await low $tval (critical)";
		$worst = $ERRORS{'CRITICAL'};
	}

	# Warnings
	if ($hi_rd_warn > -1 && $rval > $hi_rd_warn) {
		push @badlist, 
			"$dv reads high $rval (warning)";
		$worst = $ERRORS{'WARNING'} if $worst < $ERRORS{'CRITICAL'};
	}
	if ($hi_wt_warn > -1 && $wval > $hi_wt_warn) {
		push @badlist, 
			"$dv writes high $wval (warning)";
		$worst = $ERRORS{'WARNING'} if $worst < $ERRORS{'CRITICAL'};
	}
	if ($hi_tt_warn > -1 && $tval > $hi_tt_warn) {
		push @badlist, 
			"$dv await high $tval (warning)";
		$worst = $ERRORS{'WARNING'} if $worst < $ERRORS{'CRITICAL'};
	}
	if ($lo_rd_warn > -1 && $rval < $lo_rd_warn) {
		push @badlist, 
			"$dv reads low $rval (warning)";
		$worst = $ERRORS{'WARNING'} if $worst < $ERRORS{'CRITICAL'};
	}
	if ($lo_wt_warn > -1 && $wval < $lo_wt_warn) {
		push @badlist, 
			"$dv writes low $wval (warning)";
		$worst = $ERRORS{'WARNING'} if $worst < $ERRORS{'CRITICAL'};
	}
	if ($lo_tt_warn > -1 && $tval < $lo_tt_warn) {
		push @badlist, 
			"$dv await low $tval (warning)";
		$worst = $ERRORS{'WARNING'} if $worst < $ERRORS{'CRITICAL'};
	}
}

# Plugin output
$worst == $ERRORS{'OK'} ? 
	print "iostat OK @goodlist" : 
	print "@badlist";

# Kilobytes?
$kb ? 
	print " kb/s" :
	print " bk/s";

# Performance? 
print " |"."@perflist"."\n" if $perf;

print "Exit status: $worst\n" if $debug;

exit ($worst);

# Usage sub
sub print_usage () {
        print "Usage: $PROGNAME 
	[-r <low:high>] (read warning kb/s)  
	[-R <low:high>] (read critical kb/s) 
	[-w <low:high>] (write warning kb/s)
	[-W <low:high>] (write critical kb/s) 
	[-t <low:high>] (await warning kb/s)
	[-T <low:high>] (await critical kb/s) 
	[-k] (report in kilobytes)
	[-d <devices list>]
	[-p] (output Nagios performance data)
	[-s <skip devices list>] 
	[-D] (debug) [-h] (help) [-V] (Version)\n";
}

# Help sub
sub print_help () {
        print_revision($PROGNAME,'$Revision: 1.4 $');
        print "Copyright (c) 2004 Harper Mann

Perl device i/o check plugin for Nagios

";
        print_usage();
        print "
-r, --readwarning
   Read warning <low:high> values are float (Ex: -r 0.0:100.5 )
-R, --readcritical
   Read critical <low:high> values are float
-w, --writewarning
   Write warning <low:high> values are float
-W, --writecritical
   Write Critical <low:high> values are float
-t, --waitwarning
   Atime warning <low:high> values are float
-T, --waitcritical
   Atime Critical <low:high> values are float
-d, --devices=STRING
   Device(s) to check.  Comma separated (skip is ignored)
-s, --skip=STRING
   Device(s) to skip.  Comma separated
-k, --kilobytes
   Report in Kilobytes / sec instead of blocks per second
-D, --debug
   Turn on debugging.  (Verbose)
-p, --performance
   Report Nagios performance data after the ouput string
-h, --help
   Print help
-V, --Version
   Print version of plugin
";
#    support();
}
